# Copyright 2021 nakamura_yuki
# 
# This program comes from Qiita article
# URL: https://qiita.com/vpcf/items/b680f504cfe8b6a64222
#  

import struct
import numpy as np
from itertools import repeat


def _set_table(section5):
    max_level = struct.unpack_from('>H', section5, 15)[0]
    table = (
        -10, # define representative of level 0　(Missing Value)
        *struct.unpack_from('>'+str(max_level)+'H', section5, 18)
    )
    return np.array(table, dtype=np.int16)

def _decode_runlength(code, hi_level):
    for raw in code:
        if raw <= hi_level:
            level = raw
            pwr = 0
            yield level
        else:
            length = (0xFF - hi_level)**pwr * (raw - (hi_level + 1))
            pwr += 1
            yield from repeat(level, length)

def load_jmara_grib2(file):
    r'''気象庁解析雨量やレーダー雨量を返す関数

    欠損値は負の値として表現される

    Parameters
    --------
    file: `str`
        file path 
        ファイルのPATH

    Returns
    -------
    rain: `numpy.ma.MaskedArray`
        単位 (mm)

    Notes
    -----
    ``jma_rain_lat`` , ``jma_rain_lon`` はそれぞれ返り値に対応する
    `np.ndarray` 型の緯度経度である。
    '''
    with open(file, 'rb') as f:
        binary = f.read()
    
    len_ = {'sec0':16, 'sec1':21, 'sec3':72, 'sec4':82, 'sec6':6}
    
    end4 = len_['sec0'] + len_['sec1'] + len_['sec3'] + len_['sec4'] - 1
    len_['sec5'] = struct.unpack_from('>I', binary, end4+1)[0]
    section5 = binary[end4:(end4+len_['sec5']+1)]
    power = section5[17]
    # print(power)
    
    end6 = end4 + len_['sec5'] + len_['sec6']
    len_['sec7'] = struct.unpack_from('>I', binary, end6+1)[0]
    section7 = binary[end6:(end6+len_['sec7']+1)]
    
    highest_level = struct.unpack_from('>H', section5, 13)[0]
    level_table = _set_table(section5)
    decoded = np.fromiter(
        _decode_runlength(section7[6:], highest_level), dtype=np.int16
    ).reshape((3360, 2560))
    
    # convert level to representative
    return np.ma.masked_less((level_table[decoded]/(10**power))[::-1, :], 0)


jma_rain_lat = np.linspace(48, 20, 3360, endpoint=False)[::-1] - 1/80/1.5 / 2
jma_rain_lon = np.linspace(118, 150, 2560, endpoint=False) + 1/80 / 2



# Copyright (c) 2021, NakaMetPy Develoers.
# Distributed under the terms of the BSD 3-Clause License.
# SPDX-License-Identifier: BSD-3-Clause
#
# Original source lisence:
# Copyright (c) 2015,2016,2017,2018 MetPy Developers.
#
"""Tools for calculating with weather data."""

# What do we want to pull into the top-level namespace?
# import os
import sys
# import warnings

if sys.version_info < (3,):
    raise ImportError(
        """You are running NakaMetPy 0.1 or greater on Python 2.
        NakaMetPy does not work on Python 2.
        """)

# Must occur before below imports
# warnings.filterwarnings('ignore', 'numpy.dtype size changed')
# os.environ['PINT_ARRAY_PROTOCOL_FALLBACK'] = '0'

# from nakametpy._version import get_version  # noqa: E402
from ._version import get_version  # noqa: E402
__version__ = get_version()
del get_version



from setuptools import setup, find_packages

# install_requires = [
#     'numpy>=1.17.0',
#     'matplotlib>=3.0.0',
#     'nose>=1.2.0',]

def _requires_from_file(filename):
    return open(filename).read().splitlines()


packages = ['src', 'tests']


console_scripts = [
    # 'sample_lib_cli=sample_lib_cli.call:main',
]


setup(
    name='nakametpy',
    python_requires='>=3.6.0',
    version='2021.7.0',
    description='Meteorological modules for calculation and colormap.',
    # packages=packages,
    packages=find_packages('src'),
    # install_requires=install_requires,
    package_dir={"": "src"}, 
    install_requires=_requires_from_file('requirements.txt'),
    url = 'https://github.com/muchojp/NakaMetPy',
    author = 'muchojp',
    author_email = 'contact.muchiwo@gmail.com',
    license='BSD 3-clause',
    # entry_points={'console_scripts': console_scripts},
    # long_description=open('README.md', encoding='UTF-8').read(),
    long_description=open('README.md', encoding='UTF-8').read(),
    long_description_content_type="text/markdown",
)
